@@ -2,7 +2,7 @@
 # This file is part of the NEAT Project suite of libraries
 
 import re
-from neattext.pattern_data import EMAIL_REGEX,NUMBERS_REGEX,PHONE_REGEX,SPECIAL_CHARACTERS_REGEX,EMOJI_REGEX,URL_PATTERN,CURRENCY_REGEX,CURRENCY_SYMB_REGEX,STOPWORDS,DATE_REGEX
+from neattext.pattern_data import EMAIL_REGEX,NUMBERS_REGEX,PHONE_REGEX,SPECIAL_CHARACTERS_REGEX,EMOJI_REGEX,URL_PATTERN,CURRENCY_REGEX,CURRENCY_SYMB_REGEX,STOPWORDS,DATE_REGEX,MOST_COMMON_PUNCT_REGEX
 from neattext import TextFrame
 from collections import defaultdict
 from heapq import nlargest
@@ -25,12 +25,33 @@ def remove_phone_numbers(text):
 	result = re.sub(PHONE_REGEX,"",text)
 	return result
 
-def remove_puncts(text):
-	"""Returns A String with punctuations remove"""
+def remove_punctuations(text,most_common=True):
+	"""Returns A String with punctuations remove
+	Params:
+		most_common:boolen(True/False)
+		use the most common punctuations eg (,.?!&-_`"')
+
+	"""
 	PUNCT_REGEX = re.compile(r"""[!"&'()*,-./:;?@[\\]^_`{|}]""")
-	result = re.sub(PUNCT_REGEX,"",text)
+	MOST_COMMON_PUNCT_REGEX = re.compile(r"""[!"&',-.;?_`]""")
+	if most_common == True:
+		result = re.sub(MOST_COMMON_PUNCT_REGEX,"",text)
+	else:
+		result = re.sub(PUNCT_REGEX,"",text)
 	return result
 
+def remove_puncts(text,most_common=True):
+	"""Returns A String with punctuations remove
+	Params:
+		most_common:boolen(True/False)
+		use the most common punctuations eg (,.?!&-_`"')
+
+	"""
+	if most_common:
+		result = re.sub(MOST_COMMON_PUNCT_REGEX,"",text)
+	else:
+		result = str(text).translate(str.maketrans('','',string.punctuation))
+	return result
 
 def remove_special_characters(text):
 	"""Returns A String with the specified characters removed """
@@ -44,7 +65,7 @@ def remove_emojis(text):
 
 def remove_stopwords(text):
 	"""Returns A String with the stopwords removed """
-	result = [word for word in text.split() if word not in STOPWORDS]
+	result = [word for word in text.split() if word.lower() not in STOPWORDS]
 	return ' '.join(result)
 
 def remove_urls(text):
@@ -72,6 +93,25 @@ def remove_dates(text):
 		result = re.sub(DATE_REGEX,"",text)
 		return result
 
+def remove_non_ascii(text):
+		"""Returns A String with Non ASCII removed"""
+		import unicodedata
+		result = unicodedata.normalize('NFKD',text).encode('ascii','ignore').decode('utf-8','ignore')
+		return result
+
+
+def remove_multiple_spaces(text):
+	"""Returns A String with Multiple Whitespaces removed"""
+	result = re.sub('\\s{2,}',' ',text)
+	return result
+
+def remove_bad_quotes(text):
+	"""Returns a string with bad quotes removed"""
+	result = re.sub('[‘’“”…]',' ', text)
+	return result 
+
+
+
 def extract_emails(text):
 	"""Returns the emails extracted """
 	result = re.findall(EMAIL_REGEX,text)
@@ -128,25 +168,105 @@ def extract_html_tags(text):
 		return result
 
 
-def clean_text(text,preserve=False):
-	"""Clean Entire Text"""
-	if preserve == False:
-		email_result = re.sub(EMAIL_REGEX,"",text)
-		phone_result = re.sub(PHONE_REGEX,"",email_result)
-		number_result = re.sub(NUMBERS_REGEX,"",phone_result)
-		url_result = re.sub(URL_PATTERN,"",number_result)
-		emoji_result = re.sub(EMOJI_REGEX,"",url_result)
-		special_char_result = re.sub(SPECIAL_CHARACTERS_REGEX,"",emoji_result)
-		final_result = special_char_result.lower()
-	else:
-		email_result = re.sub(EMAIL_REGEX,"<EMAIL>",text)
-		phone_result = re.sub(PHONE_REGEX,"<PHONENUMBER>",email_result)
-		number_result = re.sub(NUMBERS_REGEX,"<NUMBERS>",phone_result)
-		url_result = re.sub(URL_PATTERN,"<URL>",number_result)
-		emoji_result = re.sub(EMOJI_REGEX,"<EMOJI>",url_result)
-		final_result = emoji_result.lower()
 
-	return final_result
+def clean_text(text,puncts=False,stopwords=True,urls=False,
+	emails=False,numbers=False,emojis=True,special_char=False,
+	phone_num=False,non_ascii=False,multiple_whitespaces=True,contractions=False,currency_symbols=False):
+	"""
+	Clean entire text
+
+	>>> clean_text(mytext,puncts=True)
+		
+	Parameters
+	----------
+	text
+
+	puncts:Boolean(True/False) default is True
+	remove punctuations.
+	
+	stopwords:Boolean(True/False) default is False
+	remove stopwords
+
+	urls:Boolean(True/False) default is True
+	remove punctuations
+
+	emails:Boolean(True/False) default is True
+	remove emails
+
+	emojis:Boolean(True/False) default is True
+	remove emojis
+
+	numbers:Boolean(True/False) default is False
+	remove numbers
+
+	phone_num:Boolean(True/False) default is False
+	remove phone numbers
+
+	non_ascii:Boolean(True/False) default is False
+	remove non ascii characters
+
+	multiple_whitespaces:Boolean(True/False) default is False
+	remove multiple whitespaces
+
+	contractions:Boolean(True/False) default is False
+	fix contractions
+
+	currency_symbols:Boolean(True/False) default is False
+	remove currency symbols
+
+	Returns
+	-------
+	string
+	
+	"""
+	
+	if puncts:
+		text = remove_punctuations(text)
+	if stopwords:
+		text = remove_stopwords(text)
+	if emails:
+		text = remove_emails(text)
+	if phone_num:
+		text = remove_phone_numbers(text)
+	if numbers:
+		text = remove_numbers(text)
+	if urls: 
+		text = remove_urls(text)
+	if emojis:
+		text = remove_emojis(text)
+	if non_ascii:
+		text = remove_non_ascii(text)	
+	if multiple_whitespaces:
+		text = remove_multiple_spaces(text)
+	if contractions:
+		text = fix_contractions(text)
+	if currency_symbols:
+		text = remove_currency_symbols(text)
+	if special_char:
+		text = remove_special_characters(text)
+		
+	return text.lower()	
+
+
+# def clean_text(text,preserve=False):
+# 	"""Clean Entire Text"""
+# 	if preserve == False:
+# 		email_result = re.sub(EMAIL_REGEX,"",text)
+# 		phone_result = re.sub(PHONE_REGEX,"",email_result)
+# 		number_result = re.sub(NUMBERS_REGEX,"",phone_result)
+# 		url_result = re.sub(URL_PATTERN,"",number_result)
+# 		emoji_result = re.sub(EMOJI_REGEX,"",url_result)
+# 		special_char_result = re.sub(SPECIAL_CHARACTERS_REGEX,"",emoji_result)
+# 		final_result = special_char_result.lower()
+# 	else:
+# 		email_result = re.sub(EMAIL_REGEX,"<EMAIL>",text)
+# 		phone_result = re.sub(PHONE_REGEX,"<PHONENUMBER>",email_result)
+# 		number_result = re.sub(NUMBERS_REGEX,"<NUMBERS>",phone_result)
+# 		url_result = re.sub(URL_PATTERN,"<URL>",number_result)
+# 		emoji_result = re.sub(EMOJI_REGEX,"<EMOJI>",url_result)
+# 		final_result = emoji_result.lower()
+
+# 	return final_result
 
 def replace_emails(text,replace_with="<EMAIL>"):
 	"""Replaces the emails in the text with custom label"""
@@ -199,6 +319,11 @@ def replace_term(text,old_term,new_term):
 	result = re.sub(old_term,new_term,text)
 	return result
 
+def replace_bad_quotes(text):
+	"""Replace bad quotes with their correct once"""
+	result = re.sub('[‘’“”…]', '"',text)
+	return result 
+
 
 def read_txt(filename):
 	"""
@@ -470,6 +595,123 @@ def fix_contractions(text):
 	"""Fix contractions in a text"""
 	text = text.lower()
 	text = re.sub(r"i'm", "i am", text)
+	text = re.sub(r"aren't", "are not",text)
+	text = re.sub(r"can't", "cannot",text)
+	text = re.sub(r"can't've", "cannot have",text)
+	text = re.sub(r"'cause", "because",text)
+	text = re.sub(r"could've", "could have",text)
+	text = re.sub(r"couldn't", "could not",text)
+	text = re.sub(r"couldn't've", "could not have",text)
+	text = re.sub(r"didn't", "did not",text)
+	text = re.sub(r"doesn't", "does not",text)
+	text = re.sub(r"don't", "do not",text)
+	text = re.sub(r"hadn't", "had not",text)
+	text = re.sub(r"hadn't've", "had not have",text)
+	text = re.sub(r"hasn't", "has not",text)
+	text = re.sub(r"haven't", "have not",text)
+	text = re.sub(r"he'd", "he would",text)
+	text = re.sub(r"he'd've", "he would have",text)
+	text = re.sub(r"he'll", "he will",text)
+	text = re.sub(r"he'll've", "he will have",text)
+	text = re.sub(r"he's", "he is",text)
+	text = re.sub(r"how'd", "how did",text)
+	text = re.sub(r"how'd'y", "how do you",text)
+	text = re.sub(r"how'll", "how will",text)
+	text = re.sub(r"how's", "how is",text)
+	text = re.sub(r"I'd", "I would",text)
+	text = re.sub(r"I'd've", "I would have",text)
+	text = re.sub(r"I'll", "I will",text)
+	text = re.sub(r"I'll've", "I will have",text)
+	text = re.sub(r"I'm", "I am",text)
+	text = re.sub(r"I've", "I have",text)
+	text = re.sub(r"isn't", "is not",text)
+	text = re.sub(r"it'd", "it had",text)
+	text = re.sub(r"it'd've", "it would have",text)
+	text = re.sub(r"it'll", "it will",text)
+	text = re.sub(r"it'll've", "it will have",text)
+	text = re.sub(r"it's", "it is",text)
+	text = re.sub(r"let's", "let us",text)
+	text = re.sub(r"ma'am", "madam",text)
+	text = re.sub(r"mayn't", "may not",text)
+	text = re.sub(r"might've", "might have",text)
+	text = re.sub(r"mightn't", "might not",text)
+	text = re.sub(r"mightn't've", "might not have",text)
+	text = re.sub(r"must've", "must have",text)
+	text = re.sub(r"mustn't", "must not",text)
+	text = re.sub(r"mustn't've", "must not have",text)
+	text = re.sub(r"needn't", "need not",text)
+	text = re.sub(r"needn't've", "need not have",text)
+	text = re.sub(r"o'clock", "of the clock",text)
+	text = re.sub(r"oughtn't", "ought not",text)
+	text = re.sub(r"oughtn't've", "ought not have",text)
+	text = re.sub(r"shan't", "shall not",text)
+	text = re.sub(r"sha'n't", "shall not",text)
+	text = re.sub(r"shan't've", "shall not have",text)
+	text = re.sub(r"she'd", "she would",text)
+	text = re.sub(r"she'd've", "she would have",text)
+	text = re.sub(r"she'll", "she will",text)
+	text = re.sub(r"she'll've", "she will have",text)
+	text = re.sub(r"she's", "she is",text)
+	text = re.sub(r"should've", "should have",text)
+	text = re.sub(r"shouldn't", "should not",text)
+	text = re.sub(r"shouldn't've", "should not have",text)
+	text = re.sub(r"so've", "so have",text)
+	text = re.sub(r"so's", "so is",text)
+	text = re.sub(r"that'd", "that would",text)
+	text = re.sub(r"that'd've", "that would have",text)
+	text = re.sub(r"that's", "that is",text)
+	text = re.sub(r"there'd", "there had",text)
+	text = re.sub(r"there'd've", "there would have",text)
+	text = re.sub(r"there's", "there is",text)
+	text = re.sub(r"they'd", "they would",text)
+	text = re.sub(r"they'd've", "they would have",text)
+	text = re.sub(r"they'll", "they will",text)
+	text = re.sub(r"they'll've", "they will have",text)
+	text = re.sub(r"they're", "they are",text)
+	text = re.sub(r"they've", "they have",text)
+	text = re.sub(r"to've", "to have",text)
+	text = re.sub(r"wasn't", "was not",text)
+	text = re.sub(r"we'd", "we had",text)
+	text = re.sub(r"we'd've", "we would have",text)
+	text = re.sub(r"we'll", "we will",text)
+	text = re.sub(r"we'll've", "we will have",text)
+	text = re.sub(r"we're", "we are",text)
+	text = re.sub(r"we've", "we have",text)
+	text = re.sub(r"weren't", "were not",text)
+	text = re.sub(r"what'll", "what will",text)
+	text = re.sub(r"what'll've", "what will have",text)
+	text = re.sub(r"what're", "what are",text)
+	text = re.sub(r"what's", "what is",text)
+	text = re.sub(r"what've", "what have",text)
+	text = re.sub(r"when's", "when is",text)
+	text = re.sub(r"when've", "when have",text)
+	text = re.sub(r"where'd", "where did",text)
+	text = re.sub(r"where's", "where is",text)
+	text = re.sub(r"where've", "where have",text)
+	text = re.sub(r"who'll", "who will",text)
+	text = re.sub(r"who'll've", "who will have",text)
+	text = re.sub(r"who's", "who is",text)
+	text = re.sub(r"who've", "who have",text)
+	text = re.sub(r"why's", "why is",text)
+	text = re.sub(r"why've", "why have",text)
+	text = re.sub(r"will've", "will have",text)
+	text = re.sub(r"won't", "will not",text)
+	text = re.sub(r"won't've", "will not have",text)
+	text = re.sub(r"would've", "would have",text)
+	text = re.sub(r"wouldn't", "would not",text)
+	text = re.sub(r"wouldn't've", "would not have",text)
+	text = re.sub(r"y'all", "you all",text)
+	text = re.sub(r"y'alls", "you alls",text)
+	text = re.sub(r"y'all'd", "you all would",text)
+	text = re.sub(r"y'all'd've", "you all would have",text)
+	text = re.sub(r"y'all're", "you all are",text)
+	text = re.sub(r"y'all've", "you all have",text)
+	text = re.sub(r"you'd", "you had",text)
+	text = re.sub(r"you'd've", "you would have",text)
+	text = re.sub(r"you'll", "you you will",text)
+	text = re.sub(r"you'll've", "you you will have",text)
+	text = re.sub(r"you're", "you are",text)
+	text = re.sub(r"you've", "you have",text)
 	text = re.sub(r"he's", "he is", text)
 	text = re.sub(r"she's", "she is", text)
 	text = re.sub(r"that's", "that is", text)        
@@ -487,6 +729,25 @@ def fix_contractions(text):
 	text = re.sub(r"it's", "it is", text)
 	text = re.sub(r"couldn't", "could not", text)
 	text = re.sub(r"have't", "have not", text)
+	# text = re.sub(r"i'm", "i am", text)
+	# text = re.sub(r"he's", "he is", text)
+	# text = re.sub(r"she's", "she is", text)
+	# text = re.sub(r"that's", "that is", text)        
+	# text = re.sub(r"what's", "what is", text)
+	# text = re.sub(r"where's", "where is", text) 
+	# text = re.sub(r"\'ll", " will", text)  
+	# text = re.sub(r"\'ve", " have", text)  
+	# text = re.sub(r"\'re", " are", text)
+	# text = re.sub(r"\'d", " would", text)
+	# text = re.sub(r"\'ve", " have", text)
+	# text = re.sub(r"won't", "will not", text)
+	# text = re.sub(r"don't", "do not", text)
+	# text = re.sub(r"did't", "did not", text)
+	# text = re.sub(r"can't", "can not", text)
+	# text = re.sub(r"it's", "it is", text)
+	# text = re.sub(r"couldn't", "could not", text)
+	# text = re.sub(r"have't", "have not", text)
+
 	return text
 
 
